;;; init-evil.el --- VIM dentro de emacs  -*- lexical-binding: t -*-

;; Author: kj <webmaster@outcontrol.net>
;; URL: https://git.kj2.me/kj/confi-emacs-actual

;;; Commentary:

;; En este archivo está toda la configuración relacionada
;; con el paquete Evil.
;;
;; Evil es un paquete para Emacs, que básicamente nos
;; permite tener VIM dentro de Emacs.

;;; Code:
(use-package evil
  :defer t
  :hook (after-init . evil-mode)
  :config
  ;; Configuraciones básicas
  (setq evil-default-state 'emacs) ;; Modo por defecto en emacs
  (setq evil-emacs-state-cursor 'bar) ;; Cursor de emacs state en bar
  (setq evil-normal-state-cursor 'bar) ;; Cursor de normal state en bar en lugar de box
  (evil-set-leader 'motion (kbd "SPC")) ;; Leader key
  (evil-set-undo-system 'undo-redo)

  ;; Habilitar atajos de de tecla de emacs en insert state
  (setq evil-insert-state-map (make-sparse-keymap))

  ;; Ingresar en normal state con ESC desde insert state o desde Emacs state
  (evil-define-key 'insert 'global (kbd "<escape>") 'evil-normal-state)
  ;; (evil-define-key 'emacs 'global (kbd "<escape>") 'evil-normal-state)
  ;; Cambiar Insert state por emacs state.
  ;; (defalias 'evil-insert-state 'evil-emacs-state)

  ;; Variales locales (todos los estados o solo los estados de VIM)
  (setq all-states '(normal insert visual replace operator motion emacs))
  (setq vim-states '(normal insert visual replace operator motion))

  ;; Cambiar entre modo emacs y evil (Mi pripio evil toggle)
  (evil-define-key vim-states 'global (kbd "C-'") 'evil-emacs-state)
  (evil-define-key 'emacs 'global (kbd "C-'") 'evil-normal-state)

  ;; Cambiar C-z a deshacer
  (evil-define-key all-states 'global (kbd "C-z") 'undo)

  ;; Cambiar C-r a ripgrep y U hasta hacer redo
  (evil-define-key vim-states 'global (kbd "C-r") 'counsel-rg)
  (evil-define-key 'normal 'global (kbd "U") 'undo-redo)

  ;; Arreglar yank-pop en normal mode
  (evil-define-key 'normal 'global (kbd "M-y") 'counsel-yank-pop)

  ;; Re-bindear keys a como funcionan en emacs.
  (evil-define-key 'normal 'global (kbd "C-p") 'previous-line)
  (evil-define-key 'normal 'global (kbd "C-n") 'next-line)
  (evil-define-key 'normal 'global (kbd "C-e") 'move-end-of-line)

  ;; Navegación entre frames
  (evil-define-key 'motion 'global (kbd "<leader>o") 'other-frame)

  ;; Atajos con leader key para frames
  (evil-define-key 'motion 'global (kbd "<leader>0") 'delete-window)
  (evil-define-key 'motion 'global (kbd "<leader>1") 'delete-other-windows)
  (evil-define-key 'motion 'global (kbd "<leader>2") 'split-window-below)
  (evil-define-key 'motion 'global (kbd "<leader>3") 'split-window-right)
  (evil-define-key 'motion 'global (kbd "<leader>k") 'kill-buffer)
  (evil-define-key 'motion 'global (kbd "<leader>wu") 'winner-undo)
  (evil-define-key 'motion 'global (kbd "<leader>wr") 'winner-redo)

  ;; Cambios de modos
  (evil-define-key 'normal 'global (kbd "<leader>mh") 'html-mode)
  (evil-define-key 'normal 'global (kbd "<leader>mj") 'js-mode)
  (evil-define-key 'normal 'global (kbd "<leader>mp") 'php-mode)
  (evil-define-key 'normal 'global (kbd "<leader>mr") 'rust-mode)
  (evil-define-key 'normal 'global (kbd "<leader>mw") 'web-mode)

  ;; Otros atajos
  (evil-define-key 'motion 'global (kbd "<leader>b") 'ivy-switch-buffer)          ;; Cambiar de buffer
  (evil-define-key 'motion 'global (kbd "<leader>p") 'counsel-projectile)         ;; Archivos del proyecto (o abrir proyecto si no hay alguno)
  (evil-define-key 'motion 'global (kbd "<leader>cp") 'projectile-switch-project) ;; Cambiar proyecto
  (evil-define-key 'motion 'global (kbd "<leader>ff") 'counsel-find-file)         ;; Abrir archivo

  ;; Iniciar en normal state si es un lenguaje de programación u org-mode
  ;; (evil-set-initial-state 'prog-mode 'normal)
  ;; (evil-set-initial-state 'org-mode 'normal)
  ;; (evil-set-initial-state 'dashboard-mode 'normal)
  (evil-set-initial-state 'dired 'normal)
  )

(provide 'init-evil)
;;; init-evil.el ends here
